@@ -331,19 +331,22 @@ void Field::get_value_by_opt_type(wxString& str, const bool check_value/* = true
         if ((m_opt.type == coFloatOrPercent || m_opt.type == coFloatsOrPercents) && !str.IsEmpty() &&  str.Last() != '%')
         {
             double val = 0.;
+
+            bool is_na_value = m_opt.nullable && str == na_value();
+
             const char dec_sep = is_decimal_separator_point() ? '.' : ',';
             const char dec_sep_alt = dec_sep == '.' ? ',' : '.';
             // Replace the first incorrect separator in decimal number.
-            if (str.Replace(dec_sep_alt, dec_sep, false) != 0)
+            if (!is_na_value && str.Replace(dec_sep_alt, dec_sep, false) != 0)
                 set_value(str, false);
 
-
             // remove space and "mm" substring, if any exists
             str.Replace(" ", "", true);
             str.Replace("m", "", true);
 
-            if (!str.ToDouble(&val))
-            {
+            if (is_na_value) {
+                val = ConfigOptionFloatsOrPercentsNullable::nil_value().value;
+            } else if (!str.ToDouble(&val)) {
                 if (!check_value) {
                     m_value.clear();
                     break;
@@ -453,8 +456,11 @@ void TextCtrl::BUILD() {
     case coFloatsOrPercents: {
 		const auto val =  m_opt.get_default_value<ConfigOptionFloatsOrPercents>()->get_at(m_opt_idx);
         text_value = double_to_string(val.value);
-        if (val.percent)
+        if (val.percent) {
             text_value += "%";
+        }
+
+        m_last_meaningful_value = text_value;
         break;
 	}
 	case coPercent:
