@@ -29,6 +29,9 @@
 #include "libslic3r/GCode/GCodeProcessor.hpp"
 #include "Jobs/Job.hpp"
 #include "Jobs/Worker.hpp"
+#include "libslic3r/GCode/ThumbnailData.hpp"
+#include "slic3r/GUI/Camera.hpp"
+#include "slic3r/Utils/PrintHost.hpp"
 
 class wxString;
 
@@ -55,6 +58,7 @@ namespace UndoRedo {
 namespace GUI {
 
 wxDECLARE_EVENT(EVT_SCHEDULE_BACKGROUND_PROCESS, SimpleEvent);
+wxDECLARE_EVENT(EVT_REGENERATE_BED_THUMBNAILS, SimpleEvent);
 
 class MainFrame;
 class GLCanvas3D;
@@ -64,6 +68,7 @@ struct Camera;
 class GLToolbar;
 class UserAccount;
 class PresetArchiveDatabase;
+enum class ArrangeSelectionMode;
 
 class Plater: public wxPanel
 {
@@ -92,10 +97,16 @@ public:
     Sidebar& sidebar();
     const Model& model() const;
     Model& model();
-    const Print& fff_print() const;
-    Print& fff_print();
-    const SLAPrint& sla_print() const;
-    SLAPrint& sla_print();
+    //const Print& fff_print() const;
+    //Print& fff_print();
+    //const SLAPrint& sla_print() const;
+    //SLAPrint& sla_print();
+    
+    Print& active_fff_print();
+    SLAPrint& active_sla_print();
+
+    std::vector<std::unique_ptr<Print>>& get_fff_prints();
+    const std::vector<GCodeProcessorResult>& get_gcode_results() const;
 
     void new_project();
     void load_project();
@@ -110,6 +121,8 @@ public:
     void convert_gcode_to_ascii();
     void convert_gcode_to_binary();
     void reload_print();
+    void object_list_changed();
+    void generate_thumbnail(ThumbnailData& data, unsigned int w, unsigned int h, const ThumbnailsParams& thumbnail_params, Camera::EType camera_type);
 
     std::vector<size_t> load_files(const std::vector<boost::filesystem::path>& input_files, bool load_model = true, bool load_config = true, bool imperial_units = false);
     // To be called when providing a list of files to the GUI slic3r on command line.
@@ -203,7 +216,14 @@ public:
 
     void apply_cut_object_to_model(size_t init_obj_idx, const ModelObjectPtrs& cut_objects);
 
+    void with_mocked_fff_background_process(
+        Print &print,
+        GCodeProcessorResult &result,
+        const int bed_index,
+        const std::function<void()> &callable
+    );
     void export_gcode(bool prefer_removable);
+    void export_all_gcodes(bool prefer_removable);
     void export_stl_obj(bool extended = false, bool selection_only = false);
     bool export_3mf(const boost::filesystem::path& output_path = boost::filesystem::path());
     void reload_from_disk();
@@ -228,7 +248,11 @@ public:
     void send_gcode();
     void send_gcode_inner(DynamicPrintConfig* physical_printer_config);
 	void eject_drive();
+
+    std::optional<PrintHostJob> get_connect_print_host_job();
     void connect_gcode();
+    void connect_gcode_all();
+    void printables_to_connect_gcode(const std::string& url);
     std::string get_upload_filename();
 
     void take_snapshot(const std::string &snapshot_name);
@@ -265,6 +289,7 @@ public:
     void update_menus();
     void show_action_buttons(const bool is_ready_to_slice) const;
     void show_action_buttons() const;
+    void show_autoslicing_action_buttons() const;
 
     wxString get_project_filename(const wxString& extension = wxEmptyString) const;
     void set_project_filename(const wxString& filename);
@@ -279,9 +304,10 @@ public:
     GLCanvas3D* get_current_canvas3D();
 
     void render_sliders(GLCanvas3D& canvas);
-    
+
     void arrange();
-    void arrange(Worker &w, bool selected);
+    void arrange_current_bed();
+    void arrange(Worker &w, const ArrangeSelectionMode &selected);
 
     void set_current_canvas_as_dirty();
     void unbind_canvas_event_handlers();
@@ -360,9 +386,6 @@ public:
     NotificationManager* get_notification_manager();
     const NotificationManager* get_notification_manager() const;
 
-    PresetArchiveDatabase* get_preset_archive_database();
-    const PresetArchiveDatabase* get_preset_archive_database() const;
-
     UserAccount* get_user_account();
     const UserAccount* get_user_account() const;
 
@@ -437,6 +460,12 @@ public:
     wxMenu* multi_selection_menu();
 
 private:
+    std::optional<fs_path> get_default_output_file();
+    std::optional<wxString> check_output_path_has_error(const boost::filesystem::path& path) const;
+    std::optional<fs_path> get_output_path(const std::string &start_dir, const fs_path &default_output_file);
+    std::optional<fs_path> get_multiple_output_dir(const std::string &start_dir);
+
+    void export_gcode_to_path(const fs_path &output_path, const std::function<void(bool)> &export_callback);
     void reslice_until_step_inner(int step, const ModelObject &object, bool postpone_error_messages);
 
     struct priv;
