@@ -31,7 +31,7 @@
 #include "slic3r/GUI/GUI.hpp"
 #include "slic3r/GUI/I18N.hpp"
 #include "slic3r/GUI/UpdateDialogs.hpp"
-#include "slic3r/GUI/ConfigWizard.hpp"
+#include "slic3r/Utils/PresetUpdaterWrapper.hpp"
 #include "slic3r/GUI/GUI_App.hpp"
 #include "slic3r/GUI/Plater.hpp"
 #include "slic3r/GUI/format.hpp"
@@ -74,23 +74,7 @@ void copy_file_fix(const fs::path &source, const fs::path &target)
 	static constexpr const auto perms = fs::owner_read | fs::owner_write | fs::group_read | fs::others_read;
 	fs::permissions(target, perms);
 }
-std::string escape_string_url(const std::string& unescaped)
-{
-	std::string ret_val;
-	CURL* curl = curl_easy_init();
-	if (curl) {
-		char* decoded = curl_easy_escape(curl, unescaped.c_str(), unescaped.size());
-		if (decoded) {
-			ret_val = std::string(decoded);
-			curl_free(decoded);
-		}
-		curl_easy_cleanup(curl);
-	}
-	return ret_val;
 }
-}
-
-wxDEFINE_EVENT(EVT_CONFIG_UPDATER_SYNC_DONE, wxCommandEvent);
 
 struct Update
 {
@@ -170,18 +154,13 @@ struct PresetUpdater::priv
 {
 	std::vector<Index> index_db;
 
-	bool enabled_version_check;
 	bool enabled_config_update;
-	std::string version_check_url;
 
 	fs::path cache_path;
 	fs::path cache_vendor_path;
 	fs::path rsrc_path;
 	fs::path vendor_path;
 
-	bool cancel;
-	std::thread thread;
-
 	bool has_waiting_updates { false };
 	Updates waiting_updates;
 
@@ -190,16 +169,16 @@ struct PresetUpdater::priv
 	void set_download_prefs(const AppConfig *app_config);
 	void prune_tmps() const;
 	void clear_cache_vendor() const;
-	void sync_config(const VendorMap& vendors, const GUI::ArchiveRepository* archive);
+	void sync_config(const VendorMap& vendors, const ArchiveRepository* archive, PresetUpdaterUIStatus* ui_status);
 
 	void check_install_indices() const;
 	Updates get_config_updates(const Semver& old_slic3r_version) const;
-	bool perform_updates(Updates &&updates, const SharedArchiveRepositoryVector& repositories, bool snapshot = true) const;
+	bool perform_updates(Updates &&updates, const SharedArchiveRepositoryVector& repositories, PresetUpdaterUIStatus* ui_status, bool snapshot = true) const;
 	void set_waiting_updates(Updates u);
 	// checks existence and downloads resource to cache
-	void get_missing_resource(const GUI::ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini) const;
+	void get_missing_resource(const ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini, PresetUpdaterUIStatus* ui_status) const;
 	// checks existence and downloads resource to vendor or copy from cache to vendor
-	void get_or_copy_missing_resource(const GUI::ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini) const;
+	void get_or_copy_missing_resource(const ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini, PresetUpdaterUIStatus* ui_status) const;
 	void update_index_db();
 };
 
@@ -208,7 +187,7 @@ PresetUpdater::priv::priv()
 	, cache_vendor_path(cache_path / "vendor")
 	, rsrc_path(fs::path(resources_dir()) / "profiles")
 	, vendor_path(fs::path(Slic3r::data_dir()) / "vendor")
-	, cancel(false)
+	//, cancel(false)
 {
 	set_download_prefs(GUI::wxGetApp().app_config);
 	// Install indicies from resources. Only installs those that are either missing or older than in resources.
@@ -225,8 +204,6 @@ void PresetUpdater::priv::update_index_db()
 // Pull relevant preferences from AppConfig
 void PresetUpdater::priv::set_download_prefs(const AppConfig *app_config)
 {
-	enabled_version_check = app_config->get("notify_release") != "none";
-	version_check_url = app_config->version_check_url();
 	enabled_config_update = app_config->get_bool("preset_update") && !app_config->legacy_datadir();
 }
 
@@ -253,7 +230,7 @@ void PresetUpdater::priv::clear_cache_vendor() const
 }
 
 // gets resource to cache/<vendor_name>/
-void PresetUpdater::priv::get_missing_resource(const GUI::ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini) const
+void PresetUpdater::priv::get_missing_resource(const ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini, PresetUpdaterUIStatus* ui_status) const
 {
 	assert(!filename.empty() && !vendor.empty());
 	//if (filename.empty() || vendor.empty()) {
@@ -282,13 +259,12 @@ void PresetUpdater::priv::get_missing_resource(const GUI::ArchiveRepository* arc
 	if (!fs::exists(file_in_cache.parent_path()))
 		fs::create_directory(file_in_cache.parent_path());
 
-	//std::string escaped_filename = escape_string_url(filename);
 	const std::string resource_subpath = GUI::format("%1%/%2%",vendor, filename);
-	archive->get_file(resource_subpath, file_in_cache, repository_id_from_ini);
+	archive->get_file(resource_subpath, file_in_cache, repository_id_from_ini, ui_status);
 	return;
 }
 // gets resource to vendor/<vendor_name>/
-void PresetUpdater::priv::get_or_copy_missing_resource(const GUI::ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini) const
+void PresetUpdater::priv::get_or_copy_missing_resource(const ArchiveRepository* archive, const std::string& vendor, const std::string& filename, const std::string& repository_id_from_ini, PresetUpdaterUIStatus* ui_status) const
 {
 	assert(!filename.empty() && !vendor.empty());
 
@@ -313,9 +289,8 @@ void PresetUpdater::priv::get_or_copy_missing_resource(const GUI::ArchiveReposit
 	if (!fs::exists(file_in_cache)) { 
 		BOOST_LOG_TRIVIAL(info) << "Downloading resources missing in cache directory: " << vendor << " / " << filename;
 
-		//std::string escaped_filename = escape_string_url(filename);
 		const std::string resource_subpath = GUI::format("%1%/%2%", vendor, filename);
-		archive->get_file(resource_subpath, file_in_vendor, repository_id_from_ini);
+		archive->get_file(resource_subpath, file_in_vendor, repository_id_from_ini, ui_status);
 		return;
 	}
 	BOOST_LOG_TRIVIAL(debug) << "Copiing: " << file_in_cache << " to " << file_in_vendor;
@@ -324,19 +299,22 @@ void PresetUpdater::priv::get_or_copy_missing_resource(const GUI::ArchiveReposit
 
 // Download vendor indices. Also download new bundles if an index indicates there's a new one available.
 // Both are saved in cache.
-void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::ArchiveRepository* archive_repository)
+void PresetUpdater::priv::sync_config(const VendorMap& vendors, const ArchiveRepository* archive_repository, PresetUpdaterUIStatus* ui_status)
 {
 	BOOST_LOG_TRIVIAL(info) << "Syncing configuration cache";
 
 	if (!enabled_config_update) { return; }
 
+    assert(ui_status);
+    ui_status->set_target(archive_repository->get_manifest().id + " archive");
+    
 	// Download profiles archive zip
 	fs::path archive_path(cache_path / "vendor_indices.zip");
-	if (!archive_repository->get_archive(archive_path)) {
-		BOOST_LOG_TRIVIAL(error) << "Download of vedor profiles archive zip failed.";
+	if (!archive_repository->get_archive(archive_path, ui_status)) {
+		BOOST_LOG_TRIVIAL(error) << "Download of vedor profiles archive zip of " << archive_repository->get_manifest().id << " repository has failed.";
 		return;
 	}
-	if (cancel) { 
+	if (ui_status->get_canceled()) { 
 		return;
 	}
 
@@ -406,7 +384,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 	// Update vendor preset bundles if in Vendor
 	// Over all indices from the cache directory:
 	for (auto &index : index_db) {
-		if (cancel) { 
+		if (ui_status->get_canceled()) { 
 			return; 
 		}
 		auto archive_it = std::find_if(vendors_with_status.begin(), vendors_with_status.end(),
@@ -452,7 +430,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 				BOOST_LOG_TRIVIAL(error) << format("Could not load downloaded index %1% for vendor %2%: invalid index?", idx_path, vendor.name);
 				continue;
 			}
-			if (cancel)
+			if (ui_status->get_canceled())
 				return;
 		}
 
@@ -480,9 +458,9 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 		BOOST_LOG_TRIVIAL(info) << "Downloading new bundle for vendor: " << vendor.name;
 		const std::string source_subpath = GUI::format("%1%/%2%.ini", vendor.id, recommended.to_string());
 		const fs::path bundle_path = cache_path / (vendor.id + ".ini");
-		if (!archive_repository->get_file(source_subpath, bundle_path, vendor.repo_id))
+		if (!archive_repository->get_file(source_subpath, bundle_path, vendor.repo_id, ui_status))
 			continue;
-		if (cancel)
+		if (ui_status->get_canceled())
 			return;
 		// vp is fully loaded to get all resources
 		VendorProfile vp;
@@ -499,7 +477,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 				if (! res.empty()) {
 					try
 					{
-						get_missing_resource(archive_repository, vp.id, res, vendor.repo_id);
+						get_missing_resource(archive_repository, vp.id, res, vendor.repo_id, ui_status);
 					}
 					catch (const std::exception& e)
 					{
@@ -507,7 +485,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 					}
 					
 				}
-				if (cancel)
+				if (ui_status->get_canceled())
 			    	return;
 			}
 		}
@@ -539,7 +517,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 			if (!fs::exists(ini_path_in_archive)){
 				// Download recommneded to vendor - we do not have any existing ini file so we have to use archive url.
 				const std::string source_subpath = GUI::format("%1%/%2%.ini", vendor.first, recommended.to_string());
-				if (!archive_repository->get_ini_no_id(source_subpath, ini_path_in_archive))
+				if (!archive_repository->get_ini_no_id(source_subpath, ini_path_in_archive, ui_status))
 					continue;
 			} else {
 				// check existing ini version
@@ -560,7 +538,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 				if (vp.config_version != recommended) {
 					// Take url from existing ini. This way we prevent downloading files from multiple sources.
 					const std::string source_subpath = GUI::format("%1%/%2%.ini", vp.id, recommended.to_string());
-					if (!archive_repository->get_file(source_subpath, ini_path_in_archive, vp.repo_id))
+					if (!archive_repository->get_file(source_subpath, ini_path_in_archive, vp.repo_id, ui_status))
 						continue;
 				}
 			}
@@ -577,14 +555,14 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 				if (!model.thumbnail.empty()) {
 					try
 					{
-						get_missing_resource(archive_repository, vp.id, model.thumbnail, vp.repo_id);
+						get_missing_resource(archive_repository, vp.id, model.thumbnail, vp.repo_id, ui_status);
 					}
 					catch (const std::exception& e)
 					{
 						BOOST_LOG_TRIVIAL(error) << "Failed to get " << model.thumbnail << " for " << vp.id << " " << model.id << ": " << e.what();
 					}
 				}
-				if (cancel)
+				if (ui_status->get_canceled())
 					return;
 			}
 		} else if (vendor.second == VendorStatus::IN_CACHE) {
@@ -653,7 +631,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 					continue;
 				}
 				const std::string source_subpath = GUI::format("%1%/%2%.ini", vp.id, recommended_archive.to_string());
- 				if (!archive_repository->get_file(source_subpath, ini_path_in_archive, vp.repo_id)) {
+ 				if (!archive_repository->get_file(source_subpath, ini_path_in_archive, vp.repo_id, ui_status)) {
 					BOOST_LOG_TRIVIAL(error) << format("Failed to get new vendor .ini file when checking missing resources: %1%", ini_path_in_archive.string());
 					continue;
 				}
@@ -670,7 +648,7 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 				}
 				if (vp.config_version != recommended_archive) {
 					const std::string source_subpath = GUI::format("%1%/%2%.ini", vp.id, recommended_archive.to_string());
-					if (!archive_repository->get_file(source_subpath, ini_path_in_archive, vp.repo_id)) {
+					if (!archive_repository->get_file(source_subpath, ini_path_in_archive, vp.repo_id, ui_status)) {
 						BOOST_LOG_TRIVIAL(error) << format("Failed to open vendor .ini file when checking missing resources: %1%", ini_path_in_archive);
 						continue;
 					}
@@ -694,14 +672,14 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 				if (!model.thumbnail.empty()) {
 					try
 					{
-						get_missing_resource(archive_repository, vp.id, model.thumbnail, vp.repo_id);
+						get_missing_resource(archive_repository, vp.id, model.thumbnail, vp.repo_id, ui_status);
 					}
 					catch (const std::exception& e)
 					{
 						BOOST_LOG_TRIVIAL(error) << "Failed to get " << model.thumbnail << " for " << vp.id << " " << model.id << ": " << e.what();
 					}
 				}
-				if (cancel)
+				if (ui_status->get_canceled())
 					return;
 			}
 		} else if (vendor.second == VendorStatus::INSTALLED || vendor.second == VendorStatus::NEW_VERSION) {
@@ -724,14 +702,14 @@ void PresetUpdater::priv::sync_config(const VendorMap& vendors, const GUI::Archi
 					if (!res.empty()) {
 						try
 						{
-							get_or_copy_missing_resource(archive_repository, vp.id, res, vp.repo_id);
+							get_or_copy_missing_resource(archive_repository, vp.id, res, vp.repo_id, ui_status);
 						}
 						catch (const std::exception& e)
 						{
 							BOOST_LOG_TRIVIAL(error) << "Failed to get " << res << " for " << vp.id << " " << model.id << ": " << e.what();
 						}
 					}
-					if (cancel)
+					if (ui_status->get_canceled())
 						return;
 				}
 			}
@@ -962,7 +940,7 @@ Updates PresetUpdater::priv::get_config_updates(const Semver &old_slic3r_version
 	return updates;
 }
 
-bool PresetUpdater::priv::perform_updates(Updates &&updates, const SharedArchiveRepositoryVector& repositories, bool snapshot) const
+bool PresetUpdater::priv::perform_updates(Updates &&updates, const SharedArchiveRepositoryVector& repositories, PresetUpdaterUIStatus* ui_status, bool snapshot) const
 {
 	if (updates.incompats.size() > 0) {
 		if (snapshot) {
@@ -1051,7 +1029,7 @@ bool PresetUpdater::priv::perform_updates(Updates &&updates, const SharedArchive
 					{
                         auto it = std::find_if(repositories.begin(), repositories.end(), [&vp](const auto* i){ return vp.repo_id == i->get_manifest().id; });
                         if (it != repositories.end())
-                            get_or_copy_missing_resource((*it), vp.id, resource, vp.repo_id);
+                            get_or_copy_missing_resource((*it), vp.id, resource, vp.repo_id, ui_status);
                         else {
                             BOOST_LOG_TRIVIAL(error) << "Failed to prepare " << resource << " for " << vp.id << " " << model.id << ": Missing record for source with repo_id " << vp.repo_id;
                         }
@@ -1085,74 +1063,20 @@ PresetUpdater::PresetUpdater() :
 
 PresetUpdater::~PresetUpdater()
 {
-	if (p && p->thread.joinable()) {
-		// This will stop transfers being done by the thread, if any.
-		// Cancelling takes some time, but should complete soon enough.
-		p->cancel = true;
-		p->thread.join();
-	}
 }
 
-void PresetUpdater::sync(const PresetBundle *preset_bundle, wxEvtHandler* evt_handler,SharedArchiveRepositoryVector&& repositories)
-{
-	p->set_download_prefs(GUI::wxGetApp().app_config);
-	if (!p->enabled_config_update) { return; }
-
-    p->thread = std::thread([this, &vendors = preset_bundle->vendors, repositories = std::move(repositories), evt_handler]() {
-		this->p->clear_cache_vendor();
-		this->p->prune_tmps();
-        for (const GUI::ArchiveRepository* archive : repositories) {
-		    this->p->sync_config(vendors, archive);
-		}
-		wxCommandEvent* evt = new wxCommandEvent(EVT_CONFIG_UPDATER_SYNC_DONE);
-		evt_handler->QueueEvent(evt);
-    });
-}
-
-void PresetUpdater::cancel_sync()
-{
-	if (p && p->thread.joinable()) {
-		// This will stop transfers being done by the thread, if any.
-		// Cancelling takes some time, but should complete soon enough.
-		p->cancel = true;
-		p->thread.join();
-	}
-	p->cancel = false;
-}
-
-void PresetUpdater::sync_blocking(const PresetBundle* preset_bundle, wxEvtHandler* evt_handler, const SharedArchiveRepositoryVector& repositories)
+void PresetUpdater::sync_blocking(const VendorMap& vendors, const SharedArchiveRepositoryVector& repositories, PresetUpdaterUIStatus* ui_status)
 {
 	p->set_download_prefs(GUI::wxGetApp().app_config);
 	if (!p->enabled_config_update) { return; }
 
 	this->p->clear_cache_vendor();
 	this->p->prune_tmps();
-    for (const GUI::ArchiveRepository* archive : repositories) {
-	    this->p->sync_config(preset_bundle->vendors, archive);
-	}
-}
-
-void PresetUpdater::slic3r_update_notify()
-{
-	if (! p->enabled_version_check)
-		return;
-	auto* app_config = GUI::wxGetApp().app_config;
-	const auto ver_online_str = app_config->get("version_online");
-	const auto ver_online = Semver::parse(ver_online_str);
-	const auto ver_online_seen = Semver::parse(app_config->get("version_online_seen"));
-
-	if (ver_online) {
-		// Only display the notification if the version available online is newer AND if we haven't seen it before
-		if (*ver_online > Slic3r::SEMVER && (! ver_online_seen || *ver_online_seen < *ver_online)) {
-			GUI::MsgUpdateSlic3r notification(Slic3r::SEMVER, *ver_online);
-			notification.ShowModal();
-			if (notification.disable_version_check()) {
-				app_config->set("notify_release", "none");
-				p->enabled_version_check = false;
-			}
-		}
-
-		app_config->set("version_online_seen", ver_online_str);
+    for (const ArchiveRepository* archive : repositories) {
+        if (ui_status && ui_status->get_canceled()) {
+            break;
+        }
+	    this->p->sync_config(vendors, archive, ui_status);
 	}
 }
 
@@ -1175,7 +1099,7 @@ static bool reload_configs_update_gui()
 	return true;
 }
 
-PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3r_version, UpdateParams params, const SharedArchiveRepositoryVector& repositories) const
+PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3r_version, UpdateParams params, const SharedArchiveRepositoryVector& repositories, PresetUpdaterUIStatus* ui_status) const
 {
  	if (! p->enabled_config_update) { return R_NOOP; }
 
@@ -1209,7 +1133,7 @@ PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3
 
 			// This effectively removes the incompatible bundles:
 			// (snapshot is taken beforehand)
-			if (! p->perform_updates(std::move(updates), repositories) ||
+			if (! p->perform_updates(std::move(updates), repositories, ui_status) ||
 				! GUI::wxGetApp().run_wizard(GUI::ConfigWizard::RR_DATA_INCOMPAT))
 				return R_INCOMPAT_EXIT;
 
@@ -1251,7 +1175,7 @@ PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3
 			const auto res = dlg.ShowModal();
 			if (res == wxID_OK) {
 				BOOST_LOG_TRIVIAL(info) << "User wants to update...";
-				if (! p->perform_updates(std::move(updates), repositories) ||
+				if (! p->perform_updates(std::move(updates), repositories, ui_status) ||
 					! reload_configs_update_gui())
 					return R_INCOMPAT_EXIT;
 				return R_UPDATE_INSTALLED;
@@ -1262,6 +1186,11 @@ PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3
 			}
 		}
 
+		if (!wxApp::GetInstance() || ! GUI::wxGetApp().plater()) {
+			// The main thread might have start killing the UI.
+			return R_NOOP;
+		}
+
 		// regular update
 		if (params == UpdateParams::SHOW_NOTIFICATION) {
 			p->set_waiting_updates(updates);
@@ -1289,12 +1218,12 @@ PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3
 				updates_msg.emplace_back(update.vendor, update.version.config_version, update.version.comment, std::move(changelog_url), std::move(printers));
 			}
 
-			GUI::MsgUpdateConfig dlg(updates_msg, params == UpdateParams::FORCED_BEFORE_WIZARD);
+			GUI::MsgUpdateConfig dlg(updates_msg, params);
 
 			const auto res = dlg.ShowModal();
 			if (res == wxID_OK) {
 				BOOST_LOG_TRIVIAL(debug) << "User agreed to perform the update";
-				if (! p->perform_updates(std::move(updates), repositories) ||
+				if (! p->perform_updates(std::move(updates), repositories, ui_status) ||
 					! reload_configs_update_gui())
 					return R_ALL_CANCELED;
 				return R_UPDATE_INSTALLED;
@@ -1315,7 +1244,7 @@ PresetUpdater::UpdateResult PresetUpdater::config_update(const Semver& old_slic3
 	return R_NOOP;
 }
 
-bool PresetUpdater::install_bundles_rsrc_or_cache_vendor(std::vector<std::string> bundles, const SharedArchiveRepositoryVector& repositories, bool snapshot) const
+bool PresetUpdater::install_bundles_rsrc_or_cache_vendor(std::vector<std::string> bundles, const SharedArchiveRepositoryVector& repositories, PresetUpdaterUIStatus* ui_status, bool snapshot) const
 {
 	Updates updates;
 
@@ -1418,10 +1347,10 @@ bool PresetUpdater::install_bundles_rsrc_or_cache_vendor(std::vector<std::string
 		}
 	}
 
-	return p->perform_updates(std::move(updates), repositories, snapshot);
+	return p->perform_updates(std::move(updates), repositories, ui_status, snapshot);
 }
 
-void PresetUpdater::on_update_notification_confirm(const SharedArchiveRepositoryVector& repositories)
+void PresetUpdater::on_update_notification_confirm(const SharedArchiveRepositoryVector& repositories, PresetUpdaterUIStatus* ui_status)
 {
 	if (!p->has_waiting_updates)
 		return;
@@ -1439,12 +1368,12 @@ void PresetUpdater::on_update_notification_confirm(const SharedArchiveRepository
 		updates_msg.emplace_back(update.vendor, update.version.config_version, update.version.comment, std::move(changelog_url), std::move(printers));
 	}
 
-	GUI::MsgUpdateConfig dlg(updates_msg);
+	GUI::MsgUpdateConfig dlg(updates_msg, UpdateParams::SHOW_TEXT_BOX);
 
 	const auto res = dlg.ShowModal();
 	if (res == wxID_OK) {
 		BOOST_LOG_TRIVIAL(debug) << "User agreed to perform the update";
-		if (p->perform_updates(std::move(p->waiting_updates), repositories) &&
+		if (p->perform_updates(std::move(p->waiting_updates), repositories, ui_status) &&
 			reload_configs_update_gui()) {
 			p->has_waiting_updates = false;
 		}
@@ -1454,28 +1383,10 @@ void PresetUpdater::on_update_notification_confirm(const SharedArchiveRepository
 	}	
 }
 
-bool PresetUpdater::version_check_enabled() const
-{
-	return p->enabled_version_check;
-}
-
 void PresetUpdater::update_index_db()
 {
 	p->update_index_db();
 }
-void PresetUpdater::add_additional_archive(const std::string& archive_url, const std::string& download_url)
-{
-	if (std::find_if(m_additional_archives.begin(), m_additional_archives.end(), [archive_url](const std::pair<std::string, std::string>& it) { return  it.first == archive_url; }) == m_additional_archives.end()) {
-		m_additional_archives.emplace_back(archive_url, download_url);
-	}
-}
-
-void PresetUpdater::add_additional_archives(const std::vector<std::pair<std::string, std::string>>& archives)
-{
-	for (const auto& pair : archives) {
-		add_additional_archive(pair.first, pair.second);
-	}
-}
 
 }
 
